/*
   net.c -- PIKT common network routines

   $Header: /var/cvs/PIKT/pikt/src/lib/net.c,v 1.10 2007/01/25 08:57:39 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998-2007 by Robert Osterlund.

   PIKT is a trademark of the University of Chicago.  All Rights Reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   Send problem reports, bouquets and brickbats to:  pikt-bugs@pikt.org
 */

#include "common.h"
#include <strings.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <rpc/types.h>
#include <rpc/xdr.h>
#include <rpc/auth.h>
#include <rpc/svc.h>

/* attempt to find an ethernet address within a given string (e.g., a
   PIKT.conf setting or the text result of an arp command); this is far
   from perfect, and unusual, quirky 'arp <piktmaster>' output might break
   this!

   this function returns a memory block (an ethernet address as character
   text) that should eventually be free()'ed
 */
char *
get_ethaddr (char *addr)
{
	static char fnnam[] = "get_ethaddr";
	char *a;
	char *atmp;
	register char *cp;
	register int i;

	a = mystrdup(addr, fnnam);
	if ((cp = strrchr(a, ':')) && isalnum((int)*(cp+1))) {
		i = 0;
		do {
			i++;
		} while (*(cp+i) && isalnum((int)*(cp+i)));
		*(cp+i) = '\0';
		if ((cp = strchr(a, ':')) && isalnum((int)*(cp+1))) {
			i = 0;
			do {
				i++;
			} while ((cp-i >= a) && isalnum((int)*(cp-i)));
			atmp = mystrdup(cp-i+1, fnnam);
			free(a);
			a = atmp;
		} else {
			*a = '\0';
		}
	} else {
		*a = '\0';
	}
	return a;
}

/* given two text strings--e.g., an ethernet address set in PIKT.conf and
   the output of an 'arp <piktmaster>' command--attempt to match the
   ethernet addresses contained therein
 */
int
ethaddr_match (char *str1, char *str2)
{
	char *a1, *a2;
	int b1[6], b2[6];
	register int i;
	int match;

	match = TRUE;
	a2 = (char *) NULL;
	if (    (a1 = get_ethaddr(str1))
	     && *a1
	     && (sscanf(a1, "%x:%x:%x:%x:%x:%x", &b1[0], &b1[1], &b1[2], &b1[3], &b1[4], &b1[5]) == 6)
	   ) {
		if (    (a2 = get_ethaddr(str2))
		     && *a2
		     && (sscanf(a2, "%x:%x:%x:%x:%x:%x", &b2[0], &b2[1], &b2[2], &b2[3], &b2[4], &b2[5]) == 6)
		   ) {
			for (i=0; i<6; i++) {
				if (b1[i] != b2[i]) {
					match = FALSE;
					break;
				}
			}
		} else {
			match = FALSE;
		}
	} else {
		match = FALSE;
	}
	free(a1);
	FREE(a2);
	return match;
}

/* create a socket, bind it to the specified port, and return the socket
   descriptor
 */
int getsd(void) {

#if RPCPRTNUM == no

	return RPC_ANYSOCK;

#else

	int sd;
	struct sockaddr_in sin;

	sd = socket(AF_INET, SOCK_STREAM, 0);
	bzero(&sin, sizeof(sin));
	sin.sin_family = AF_INET;
	sin.sin_addr.s_addr = INADDR_ANY;
	sin.sin_port = htons(RPCPRTNUM);
	bind(sd, &sin, sizeof(sin));
	return sd;

#endif	/* RPCPRTNUM */

}
